/*
 *	NEWS-0.4.5 - RSS Feed Ticker - Copyright (C) Emmanuel Thomas-Maurin 2009-2011
 *	< manutm007@gmail.com >
 *
 * 	This program is free software: you can redistribute it and/or modify
 * 	it under the terms of the GNU General Public License as published by
 * 	the Free Software Foundation, either version 3 of the License, or
 * 	(at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 	GNU General Public License for more details.
 *
 * 	You should have received a copy of the GNU General Public License
 * 	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "news.h"

#if USE_GUI
/*
 * fill (char)url_array[NURLMAX + 1][FILENAME_MAXLEN + 1] with lines from file
 * we pass a reference to (char *)p_url[NURLMAX + 1] which has been set as
 * p_url[i] = url_array[i] and p_url[NURLMAX] = NULL
 */
int load_url_list(char **p_url)
{
	char	*listfname, listfname1[512];
#ifdef G_OS_WIN32
	char	listfname2[512];
	gchar	*win32_filename;
#endif
	FILE	*fp;
	char	*tmp;
	size_t	tmp_size = FILENAME_MAXLEN + 1;
	int	i, j;

#ifndef G_OS_WIN32
	str_n_cpy(listfname1, usr_home_dir(), 511 - 64);
	str_n_cat(listfname1, "/"URLLIST, 64);
#else
	str_n_cpy(listfname1, get_appdata_dir(), 511 - 64);
	str_n_cat(listfname1, "\\"URLLIST, 64);
	str_n_cpy(listfname2, get_progfiles_dir(), 511 - 64);
	str_n_cat(listfname2, "\\"URLLIST, 64);
#endif
	listfname = listfname1;
#ifndef G_OS_WIN32
	if ((fp = g_fopen(listfname, "rb")) != NULL) {
#else
	win32_filename = g_win32_locale_filename_from_utf8(listfname);
	if ((fp = g_fopen(win32_filename, "rb")) == NULL) {
		info_win_wait("Setting up URL list sample", 1500);
		g_free(win32_filename);
		listfname = listfname2;
		win32_filename = g_win32_locale_filename_from_utf8(listfname);
	}
	if ((fp = g_fopen(win32_filename, "rb")) != NULL) {
		g_free(win32_filename);
#endif
		tmp = malloc2(tmp_size * sizeof(char));
		i = 0;
#ifndef G_OS_WIN32
		while (getline(&tmp, &tmp_size, fp) != EOF && i < NURLMAX) {
#else
		while (fgets(tmp, tmp_size, fp) != NULL && i < NURLMAX) {
#endif
			for (j = 0; j < FILENAME_MAXLEN; j++)
				if (tmp[j] == '\n') {
					tmp[j] = '\0';
					break;
				}
			if (tmp[0] != '\0')
				str_n_cpy(p_url[i++], tmp, FILENAME_MAXLEN);
		}
		p_url[i] = NULL;
		free2(tmp);
		fclose(fp);
		return LOAD_URL_LIST_OK;
	} else {
#ifdef G_OS_WIN32
		g_free(win32_filename);
#endif
		warning("Can't load URL list:", listfname, strerror(errno), "");
		return LOAD_URL_LIST_ERROR;
	}
}

/*
 * sort list and remove duplicates
 */
void sort_url_list(char **p_url)
{
	char	*p_url2[NURLMAX + 1], *tmp, empty_str[] = "";
	int	list_len, min, i, j;

	/*
	 * copy p_url[] to p_url2[]
	 */
	for (i = 0; i < NURLMAX && p_url[i] != NULL; i++)
		p_url2[i] = p_url[i];
	list_len = i;
	/*
	 * sort array (selection sort)
	 */
	for (i = 0; i < list_len; i++) {
		min = i;
		for (j = i  + 1; j < list_len; j++) {
			if (strcmp(p_url2[min], p_url2[j]) > 0)
				min = j;
		}
		tmp = p_url2[i];
		p_url2[i] = p_url2[min];
		p_url2[min] = tmp;
	}
	/*
	 * remove duplicated and empty entries
	 * as we can have one entry with title (url + title) and the same without title
	 * (url only), we use str_without_title() to compare strings because we want to
	 * keep entries with title and remove similar entries without title
	 */
	for (i = 0; i < list_len; i++)
		for (j = i  + 1; j < list_len; j++) {
			if (strcmp(p_url2[i], p_url2[j]) == 0)
					p_url2[j] = empty_str;
			if (strcmp(str_without_title(p_url2[i]), str_without_title(p_url2[j])) == 0) {
				if (!str_has_title(p_url2[i]))
					p_url2[i] = empty_str;
				if (!str_has_title(p_url2[j]))
					p_url2[j] = empty_str;
			}
		}

	for (i = 0, j = 0; i < list_len; i++)
		if (p_url2[i] != empty_str)
			p_url[j++] = p_url2[i];
	p_url[j] = NULL;

	return;
}

int save_url_list(char **p_url)
{
	char	listfname[512], tmp[FILENAME_MAXLEN + 1];
#ifdef G_OS_WIN32
	gchar	*win32_filename;
#endif
	FILE	*fp;
	int	i, j;

#ifndef G_OS_WIN32
	str_n_cpy(listfname, usr_home_dir(), 511 - 64);
	str_n_cat(listfname, "/"URLLIST, 64);
#else
	str_n_cpy(listfname, get_appdata_dir(), 511 - 64);
	str_n_cat(listfname, "\\"URLLIST, 64);
#endif
#ifndef G_OS_WIN32
	if ((fp = g_fopen(listfname, "wb")) != NULL) {
#else
	win32_filename = g_win32_locale_filename_from_utf8(listfname);
	if ((fp = g_fopen(win32_filename, "wb")) != NULL) {
		g_free(win32_filename);
#endif
		for (i = 0, j = 0; p_url[i] != NULL && j < NURLMAX; i++) {
			str_n_cpy(tmp, p_url[i], FILENAME_MAXLEN);
			if (tmp[0] != '\0') {
				fprintf(fp, "%s\n", tmp);
				j++;
			}
		}
		fclose(fp);
		return SAVE_URL_LIST_OK;
	} else {
#ifdef G_OS_WIN32
		g_free(win32_filename);
#endif
		warning("Can't save URL list:", listfname, strerror(errno), "");
		return SAVE_URL_LIST_ERROR;
	}
}

void remove_url_from_list(char **p_url, int rank)
{
	p_url[rank][0] = '\0';
	return;
}

int str_has_title(char *str)
{
	int	i;

	for (i = 0; i < FILENAME_MAXLEN && str[i] != '\0'; i++)
		if (str[i]  == TITLE_TAG_CHAR)
			return TRUE;
	return FALSE;
}

char *str_without_title(char *str)
{
	static char	str2[64][FILENAME_MAXLEN + 1];
	static int	count = -1;
	int		i;

	count++;
	count &= 63;

	str_n_cpy(str2[count], str, FILENAME_MAXLEN);
	for (i = 0; i < FILENAME_MAXLEN && str2[count][i] != '\0'; i++)
		if (str2[count][i]  == TITLE_TAG_CHAR) {
			str2[count][i] = '\0';
			break;
		}
	return str2[count];
}
#endif
