/*
 *	NEWS-0.4.5 - RSS Feed Ticker - Copyright (C) Emmanuel Thomas-Maurin 2009-2011
 *	< manutm007@gmail.com >
 *
 * 	This program is free software: you can redistribute it and/or modify
 * 	it under the terms of the GNU General Public License as published by
 * 	the Free Software Foundation, either version 3 of the License, or
 * 	(at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 	GNU General Public License for more details.
 *
 * 	You should have received a copy of the GNU General Public License
 * 	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "news.h"

#if USE_GUI
static GtkWidget	*dialog, *entry_homefeed;
static GrEnv		*env;
static gchar		current_feed[FILENAME_MAXLEN + 1];

static gint esc_key_pressed(GtkWidget *dialog2, GdkEventKey *event, gpointer unused)
{
	unused = unused;
	if (event->keyval == GDK_Escape) {
		gtk_dialog_response(GTK_DIALOG(dialog2), GTK_RESPONSE_CANCEL);
		return TRUE;
	} else
		return FALSE;
}

static void force_quit_dialog()
{
	gtk_dialog_response(GTK_DIALOG(dialog), GTK_RESPONSE_CANCEL);
}

static gint get_current_feed(GtkWidget *curfeed_but, GdkEventKey *event, gpointer unused)
{
	env->suspend_rq = TRUE;
	curfeed_but = curfeed_but;
	event = event;
	unused = unused;
	gtk_entry_set_text(GTK_ENTRY(entry_homefeed), current_feed);
	env->suspend_rq = FALSE;
	gtk_dialog_response(GTK_DIALOG(dialog), GTK_RESPONSE_NONE);
	return TRUE;
}

/*
 * open a dialog to change all params
 * return 0 or 1 if we want to reopen the dialog
 */
int modify_params(Params *prm)
{
	GtkWidget	*table, *label[64];	/* check this is enough */
	GtkWidget	*cancel_but, *reset_but, *save_but, *apply_but, *ok_but;
	GtkWidget	*curfeed_but;
	GtkWidget	*spinbut_delay, *spinbut_shiftsize, *spinbut_winx;
	GtkWidget	*spinbut_winy, *spinbut_winw, *spinbut_winh;
	GtkWidget	*spinbut_shadowoffsetx, *spinbut_shadowoffsety;
	GtkWidget	*spinbut_shadowfx, *spinbut_rssrefresh, *spinbut_wintransparency;
	GtkWidget	*checkbut_shadow, *checkbut_alwaysontop, *checkbut_windec;
	GtkWidget	*checkbut_rsstitle, *checkbut_rssdes, *checkbut_striptags;
	GtkWidget	*checkbut_spchars;
	GtkWidget	*font_but, *fg_color_but, *bg_color_but;
	GtkWidget	*clock_font_but, *clock_fg_color_but, *clock_bg_color_but;
	GtkWidget	*rbut_box, *radio_but1, *radio_but2, *radio_but3;
	GtkWidget	*entry_newpagech, *entry_tabch, *entry_rsstitledel;
	GtkWidget	*entry_rssdesdel, *entry_linedel, *entry_cutlinedel;
	GtkWidget	*entry_openlinkcmd, *entry_openlinkargs;
	GtkObject	*adj_delay, *adj_shiftsize, *adj_winx, *adj_winy;
	GtkObject	*adj_winw, *adj_winh, *adj_shadowoffsetx, *adj_shadowfx;
	GtkObject	*adj_shadowoffsety, *adj_rssrefresh, *adj_wintransparency;
#ifdef G_OS_WIN32
	char		tmp[TMPSTR_SIZE + 1];
#endif
	char		c[2]= "c";
	gint		response;
	int		exit_status = 0, i;
	Params		*prm_bak;

	env = get_gr_env();
	str_n_cpy(current_feed, get_resource()->id, FILENAME_MAXLEN);

	gtk_window_set_keep_above(GTK_WINDOW(env->win), FALSE);

	dialog = gtk_dialog_new_with_buttons("News Preferences", GTK_WINDOW(env->win),
		 		GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT, NULL);
#ifndef G_OS_WIN32
	gtk_window_set_icon_from_file(GTK_WINDOW(dialog), NEWS_ICON, NULL);
#else
	str_n_cpy(tmp, get_progfiles_dir(), TMPSTR_SIZE - 64);
	str_n_cat(tmp, "\\"NEWS_ICON, 64);
	gtk_window_set_icon_from_file(GTK_WINDOW(dialog), tmp, NULL);
#endif
	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);

	cancel_but = gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
	reset_but = gtk_dialog_add_button(GTK_DIALOG(dialog), "Reset", GTK_RESPONSE_NO);	/* strange... */
	save_but = gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_SAVE, GTK_RESPONSE_YES);	/* strange too... */
	apply_but = gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_APPLY, GTK_RESPONSE_APPLY);
	ok_but = gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_OK, GTK_RESPONSE_OK);

	table = gtk_table_new(17, 5, FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(table), 0);
	gtk_table_set_col_spacings(GTK_TABLE(table), 5);
	gtk_container_set_border_width(GTK_CONTAINER(table), 5);	/*15*/
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), table);

	i = 0;
#define COLUMN1		0, 1
#define COLUMN2		1, 2
#define COLUMN3		2, 3
	/*
	 * delay
	 */
	label[i] = gtk_label_new("Delay <small>(Milliseconds)</small>:");
	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN1, i, i + 1);
	adj_delay = gtk_adjustment_new(prm->delay, 1, 50, 1, 5, 0);
	spinbut_delay = gtk_spin_button_new(GTK_ADJUSTMENT(adj_delay), 0.0, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), spinbut_delay, COLUMN2, i, i + 1);
	i++;
	/*
	 * shift size
	 */
	label[i] = gtk_label_new("Shift size <small>(Pixels)</small>:");
	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN1, i, i + 1);
	adj_shiftsize = gtk_adjustment_new(prm->shift_size, 1, 200, 1, 5, 0);
	spinbut_shiftsize = gtk_spin_button_new(GTK_ADJUSTMENT(adj_shiftsize), 0.0, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), spinbut_shiftsize, COLUMN2, i, i + 1);
	i++;
	/*
	 * font
	 */
	label[i] = gtk_label_new("Font <small>(Size can't be > 200)</small>:");	/* check this is up to date */
	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN1, i, i + 1);
	font_but = gtk_font_button_new_with_font(prm->font_n_s);
	gtk_table_attach_defaults(GTK_TABLE(table), font_but, COLUMN2, i, i + 1);
	i++;
	/*
	 * fg color
	 */
	label[i] = gtk_label_new("Foreground color:");
	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN1, i, i + 1);
	fg_color_but = gtk_color_button_new_with_color(&prm->fg_color);
	gtk_color_button_set_use_alpha(GTK_COLOR_BUTTON(fg_color_but), TRUE);
	gtk_color_button_set_alpha(GTK_COLOR_BUTTON(fg_color_but), prm->fg_color_alpha);
	gtk_table_attach_defaults(GTK_TABLE(table), fg_color_but, COLUMN2, i, i + 1);;
	i++;
	/*
	 * bg color
	 */
	label[i] = gtk_label_new("Background color:");
	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN1, i, i + 1);
	bg_color_but = gtk_color_button_new_with_color(&prm->bg_color);
	gtk_color_button_set_use_alpha(GTK_COLOR_BUTTON(bg_color_but), TRUE);
	gtk_color_button_set_alpha(GTK_COLOR_BUTTON(bg_color_but), prm->bg_color_alpha);
	gtk_table_attach_defaults(GTK_TABLE(table), bg_color_but, COLUMN2, i, i + 1);
	i++;
	/*
	 * win_x
	 */
	label[i] = gtk_label_new("X position <small>(Pixels)</small>:");
	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN1, i, i + 1);
	adj_winx = gtk_adjustment_new(prm->win_x, 0, env->screen_w - 20, 1, 5, 0);
	spinbut_winx = gtk_spin_button_new(GTK_ADJUSTMENT(adj_winx), 0.0, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), spinbut_winx, COLUMN2, i, i + 1);
	i++;
	/*
	 * win_y
	 */
	label[i] = gtk_label_new("Y position <small>(Pixels)</small>:");
	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN1, i, i + 1);
	adj_winy = gtk_adjustment_new(prm->win_y, 0, env->screen_h - 20, 1, 5, 0);
	spinbut_winy = gtk_spin_button_new(GTK_ADJUSTMENT(adj_winy), 0.0, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), spinbut_winy, COLUMN2, i, i + 1);
	i++;
	/*
	 * win_w
	 */
	label[i] = gtk_label_new("Width <small>(Pixels)</small>:");
	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN1, i, i + 1);
	adj_winw = gtk_adjustment_new(prm->win_w, DRWA_WIDTH_MIN, env->screen_w, 1, 5, 0);
	spinbut_winw = gtk_spin_button_new(GTK_ADJUSTMENT(adj_winw), 0.0, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), spinbut_winw, COLUMN2, i, i + 1);
	i++;
	/*
	 * win_h
	 */
	label[i] = gtk_label_new("Height <small>(Pixels - Override font size if > 0</small>):");
	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN1, i, i + 1);
	adj_winh = gtk_adjustment_new(prm->win_h, 0, env->screen_h, 1, 5, 0);
	spinbut_winh = gtk_spin_button_new(GTK_ADJUSTMENT(adj_winh), 0.0, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), spinbut_winh, COLUMN2, i, i + 1);
	i++;
	/*
	 * windec
	 */
	label[i] = gtk_label_new("Window decoration:");
	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN1, i, i + 1);
	checkbut_windec = gtk_check_button_new();
	if (prm->windec == 'y')
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbut_windec), TRUE);
	else if (prm->windec == 'n')
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbut_windec), FALSE);
	gtk_table_attach_defaults(GTK_TABLE(table), checkbut_windec, COLUMN2, i, i + 1);
	i++;
	/*
	 * always-on-top window
	 */
	label[i] = gtk_label_new("Window Always-On-Top:");
	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN1, i, i + 1);
	checkbut_alwaysontop = gtk_check_button_new();
	if (prm->always_on_top == 'y')
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbut_alwaysontop), TRUE);
	else if (prm->always_on_top == 'n')
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbut_alwaysontop), FALSE);
	gtk_table_attach_defaults(GTK_TABLE(table), checkbut_alwaysontop, COLUMN2, i, i + 1);
	i++;
	/*
	 * window transparency
	 */
	label[i] = gtk_label_new("Window transparency <small>(Actually opacity x 10):</small>");
	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN1, i, i + 1);
	adj_wintransparency = gtk_adjustment_new(prm->win_transparency * 10, 0, 10, 1, 2, 0);
	spinbut_wintransparency = gtk_spin_button_new(GTK_ADJUSTMENT(adj_wintransparency), 0.0, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), spinbut_wintransparency, COLUMN2, i, i + 1);
	i++;
	/*
	 * shadow
	 */
	label[i] = gtk_label_new("Shadow:");
	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN1, i, i + 1);
	checkbut_shadow = gtk_check_button_new();
	if (prm->shadow == 'y')
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbut_shadow), TRUE);
	else if (prm->shadow == 'n')
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbut_shadow), FALSE);
	gtk_table_attach_defaults(GTK_TABLE(table), checkbut_shadow, COLUMN2,  i, i + 1);
	i++;
	/*
	 * shadow_offset_x
	 */
	label[i] = gtk_label_new("Shadow x offset <small>(Pixels)</small>:");
	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN1, i, i + 1);
	adj_shadowoffsetx = gtk_adjustment_new(prm->shadow_offset_x, -20, 20, 1, 5, 0);
	spinbut_shadowoffsetx = gtk_spin_button_new(GTK_ADJUSTMENT(adj_shadowoffsetx), 0.0, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), spinbut_shadowoffsetx, COLUMN2, i, i + 1);
	i++;
	/*
	 * shadow_offset_y
	 */
	label[i] = gtk_label_new("Shadow y offset <small>(Pixels)</small>:");
	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN1, i, i + 1);;
	adj_shadowoffsety = gtk_adjustment_new(prm->shadow_offset_y, -20, 20, 1, 5, 0);
	spinbut_shadowoffsety = gtk_spin_button_new(GTK_ADJUSTMENT(adj_shadowoffsety), 0.0, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), spinbut_shadowoffsety, COLUMN2, i, i + 1);
	i++;
	/*
	 * shadow_fx
	 */
	label[i] = gtk_label_new("Shadow fx <small>(0 = none -> 10 = full)</small>:  ");
	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN1, i, i + 1);
	adj_shadowfx = gtk_adjustment_new(prm->shadow_fx, 0, 10, 1, 5, 0);
	spinbut_shadowfx = gtk_spin_button_new(GTK_ADJUSTMENT(adj_shadowfx), 0.0, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), spinbut_shadowfx, COLUMN2, i, i + 1);
	i++;
	/*
	 * line delimiter
	 */
	label[i] = gtk_label_new("Line delimiter:");
	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN1, i, i + 1);
	entry_linedel = gtk_entry_new();
	gtk_entry_set_max_length(GTK_ENTRY(entry_linedel), DELIMITER_MAXLEN);
	gtk_entry_set_text(GTK_ENTRY(entry_linedel), prm->line_delimiter);
	gtk_table_attach_defaults(GTK_TABLE(table), entry_linedel, COLUMN2, i, i + 1);
	i++;

	label[i] = gtk_label_new("        ");
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN3, 0, 1);

	i = 0;
#undef COLUMN1
#define COLUMN1		3, 4
#undef COLUMN2
#define COLUMN2		4, 5
	/*
	 * cut line delimiter
	 */
	label[i] = gtk_label_new("Cut line delimiter:");
	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN1, i, i + 1);
	entry_cutlinedel = gtk_entry_new();
	gtk_entry_set_max_length(GTK_ENTRY(entry_cutlinedel), DELIMITER_MAXLEN);
	gtk_entry_set_text(GTK_ENTRY(entry_cutlinedel), prm->cutline_delimiter);
	gtk_table_attach_defaults(GTK_TABLE(table), entry_cutlinedel, COLUMN2, i, i + 1);
	i++;
	/*
	 * rss refresh
	 */
	label[i] = gtk_label_new("RSS refresh <small>(Minutes)</small>:");
	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN1, i, i + 1);
	adj_rssrefresh = gtk_adjustment_new(prm->rss_refresh, 1, 60, 1, 5, 0);
	spinbut_rssrefresh= gtk_spin_button_new(GTK_ADJUSTMENT(adj_rssrefresh), 0.0, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), spinbut_rssrefresh, COLUMN2, i, i + 1);
	i++;
	/*
	 * rss title
	 */
	label[i] = gtk_label_new("RSS title:");
	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN1, i, i + 1);;
	checkbut_rsstitle = gtk_check_button_new();
	if (prm->rss_title == 'y')
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbut_rsstitle), TRUE);
	else if (prm->rss_title == 'n')
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbut_rsstitle), FALSE);
	gtk_table_attach_defaults(GTK_TABLE(table), checkbut_rsstitle, COLUMN2, i, i + 1);
	i++;
	/*
	 * rss title delimiter
	 */
	label[i] = gtk_label_new("RSS title delimiter:");
	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN1, i, i + 1);
	entry_rsstitledel = gtk_entry_new();
	gtk_entry_set_max_length(GTK_ENTRY(entry_rsstitledel), DELIMITER_MAXLEN);
	gtk_entry_set_text(GTK_ENTRY(entry_rsstitledel), prm->rss_title_delimiter);
	gtk_table_attach_defaults(GTK_TABLE(table), entry_rsstitledel, COLUMN2, i, i + 1);
	i++;
	/*
	 * rss description
	 */
	label[i] = gtk_label_new("RSS description:");
	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN1, i, i + 1);
	checkbut_rssdes = gtk_check_button_new();
	if (prm->rss_description == 'y')
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbut_rssdes), TRUE);
	else if (prm->rss_description == 'n')
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbut_rssdes), FALSE);
	gtk_table_attach_defaults(GTK_TABLE(table), checkbut_rssdes, COLUMN2, i, i + 1);
	i++;
	/*
	 * rss description delimiter
	 */
	label[i] = gtk_label_new("RSS description delimiter:");
	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN1, i, i + 1);
	entry_rssdesdel = gtk_entry_new();
	gtk_entry_set_max_length(GTK_ENTRY(entry_rssdesdel), DELIMITER_MAXLEN);
	gtk_entry_set_text(GTK_ENTRY(entry_rssdesdel), prm->rss_description_delimiter);
	gtk_table_attach_defaults(GTK_TABLE(table), entry_rssdesdel, COLUMN2, i, i + 1);
	i++;
	/*
	 * remove html tags
	 */
	label[i] = gtk_label_new("Strip html tags:");
	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN1, i, i + 1);
	checkbut_striptags = gtk_check_button_new();
	if (prm->strip_html_tags == 'y')
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbut_striptags), TRUE);
	else if (prm->strip_html_tags == 'n')
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbut_striptags), FALSE);
	gtk_table_attach_defaults(GTK_TABLE(table), checkbut_striptags, COLUMN2, i, i + 1);
	i++;
	/*
	 * homefeed / set current feed as homefeed
	 * *** button is centered -> not good like that ***
	 */
	label[i] = gtk_label_new("'Homefeed'");/*("'Homefeed' <small>(= Homepage)</small>:");*/
	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN1, i, i + 1);
	curfeed_but = gtk_button_new_with_label("Get current feed");
	gtk_table_attach(GTK_TABLE(table), curfeed_but, COLUMN1, i, i + 1, GTK_SHRINK, GTK_SHRINK, 0, 0);
	entry_homefeed = gtk_entry_new();
	gtk_entry_set_max_length(GTK_ENTRY(entry_homefeed), FILENAME_MAXLEN);
	gtk_entry_set_text(GTK_ENTRY(entry_homefeed), prm->homefeed);
	gtk_table_attach_defaults(GTK_TABLE(table), entry_homefeed, COLUMN2, i, i + 1);
	i++;
	/*
	 * "open in browser" command line
	 */
	label[i] = gtk_label_new("'Open in Browser' command line <small>(May require path)</small>:");
		/*"<small>    Application that will open active link (may require path)    \n"
		"    Most likely will invoke your favourite browser</small>\n");*/
	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN1, i, i + 1);
	entry_openlinkcmd = gtk_entry_new();
	gtk_entry_set_max_length(GTK_ENTRY(entry_openlinkcmd), FILENAME_MAXLEN);
	gtk_entry_set_text(GTK_ENTRY(entry_openlinkcmd), prm->open_link_cmd);
	gtk_table_attach_defaults(GTK_TABLE(table), entry_openlinkcmd, COLUMN2, i, i + 1);
	i++;
	/*
	 * "open in browser" args
	 */
	label[i] = gtk_label_new("'Open in Browser' optional arguments:");
	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN1, i, i + 1);
	entry_openlinkargs = gtk_entry_new();
	gtk_entry_set_max_length(GTK_ENTRY(entry_openlinkargs), FILENAME_MAXLEN);
	gtk_entry_set_text(GTK_ENTRY(entry_openlinkargs), prm->open_link_args);
	gtk_table_attach_defaults(GTK_TABLE(table), entry_openlinkargs, COLUMN2, i, i + 1);
	i++;
	/*
	 * clock
	 */
	label[i] = gtk_label_new("Clock:");
	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN1, i, i + 1);
	radio_but1 = gtk_radio_button_new_with_label(NULL, "Left");
	radio_but2 = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(radio_but1), "Right");
	radio_but3 = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(radio_but1), "None");
	rbut_box = gtk_hbox_new(TRUE, 2);
	gtk_box_pack_start(GTK_BOX(rbut_box), radio_but1, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(rbut_box), radio_but2, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(rbut_box), radio_but3, TRUE, TRUE, 0);
	if (prm->clock == 'l')
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio_but1), TRUE);
	else if (prm->clock == 'r')
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio_but2), TRUE);
	else
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio_but3), TRUE);
	gtk_table_attach_defaults(GTK_TABLE(table), rbut_box, COLUMN2, i, i + 1);
	i++;
	/*
	 * clock_font
	 */
	label[i] = gtk_label_new("Clock font <small>(Size can't be > ticker height)</small>:");
	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN1, i, i + 1);
	clock_font_but = gtk_font_button_new_with_font(prm->clock_font_n_s);
	gtk_table_attach_defaults(GTK_TABLE(table), clock_font_but, COLUMN2, i, i + 1);
	i++;
	/*
	 * clock fg color
	 */
	label[i] = gtk_label_new("Clock foreground color:");
	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN1, i, i + 1);
	clock_fg_color_but = gtk_color_button_new_with_color(&prm->clock_fg_color);
	gtk_color_button_set_use_alpha(GTK_COLOR_BUTTON(clock_fg_color_but), TRUE);
	gtk_color_button_set_alpha(GTK_COLOR_BUTTON(clock_fg_color_but), prm->clock_fg_color_alpha);
	gtk_table_attach_defaults(GTK_TABLE(table), clock_fg_color_but, COLUMN2, i, i + 1);
	i++;
	/*
	 * clock bg color
	 */
	label[i] = gtk_label_new("Clock background color:");
	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN1, i, i + 1);
	clock_bg_color_but = gtk_color_button_new_with_color(&prm->clock_bg_color);
	gtk_color_button_set_use_alpha(GTK_COLOR_BUTTON(clock_bg_color_but), TRUE);
	gtk_color_button_set_alpha(GTK_COLOR_BUTTON(clock_bg_color_but), prm->clock_bg_color_alpha);
	gtk_table_attach_defaults(GTK_TABLE(table), clock_bg_color_but, COLUMN2, i, i + 1);
	i++;
	/*
	 * special chars
	 */
	label[i] = gtk_label_new("Special characters enabled <small>(Useless if resource is URL)</small>:");
	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN1, i, i + 1);
	checkbut_spchars = gtk_check_button_new();
	if (prm->special_chars == 'y')
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbut_spchars), TRUE);
	else if (prm->special_chars == 'n')
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbut_spchars), FALSE);
	gtk_table_attach_defaults(GTK_TABLE(table), checkbut_spchars, COLUMN2, i, i + 1);
	i++;
	/*
	 * 'new page' special char
	 */
	label[i] = gtk_label_new("'New page' special char:");
	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN1, i, i + 1);
	entry_newpagech = gtk_entry_new();
	gtk_entry_set_max_length(GTK_ENTRY(entry_newpagech), 1);
	/* this does nothing - gtk_entry_set_width_chars(GTK_ENTRY(entry_newpagech), 1);*/
	c[0] = prm->new_page_char;
	gtk_entry_set_text(GTK_ENTRY(entry_newpagech), c);
	gtk_table_attach_defaults(GTK_TABLE(table), entry_newpagech, COLUMN2,  i, i + 1);
	i++;
	/*
	 * tab special char
	 */
	label[i] = gtk_label_new("Tab special char:");
	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(table), label[i], COLUMN1, i, i + 1);
	entry_tabch = gtk_entry_new();
	gtk_entry_set_max_length(GTK_ENTRY(entry_tabch), 1);
	/* this does nothing - gtk_entry_set_width_chars(GTK_ENTRY(entry_tabch), 1);*/
	c[0] = prm->tab_char;
	gtk_entry_set_text(GTK_ENTRY(entry_tabch), c);
	gtk_table_attach_defaults(GTK_TABLE(table), entry_tabch, COLUMN2, i, i + 1);
	i++;

	/* will let us know if changes have been made?*/
	prm_bak = malloc2(sizeof(Params));
	memcpy((void *)prm_bak, (const void *)prm, sizeof(Params));

	g_signal_connect(G_OBJECT(curfeed_but), "clicked", G_CALLBACK(get_current_feed), NULL);
	g_signal_connect(G_OBJECT(dialog), "key-press-event", G_CALLBACK(esc_key_pressed), NULL);
	g_signal_connect(G_OBJECT(dialog), "delete_event", G_CALLBACK(force_quit_dialog), NULL);

	gtk_widget_show_all(dialog);

	while ((response = gtk_dialog_run(GTK_DIALOG(dialog))) != GTK_RESPONSE_CANCEL) {
		env->suspend_rq = TRUE;

		if (response == GTK_RESPONSE_NO) {	/* reset */
			if (question_win("Reset parameters to default values ?")) {
				set_default_options(prm);
				env->reload_rq = TRUE;
				env->suspend_rq = FALSE;
				exit_status = 1;
				break;
			} else {
				env->suspend_rq = FALSE;
				continue;
			}
		} else if (response == GTK_RESPONSE_YES || response == GTK_RESPONSE_APPLY || \
				response == GTK_RESPONSE_OK) {
			/* delay */
			prm->delay = (int)gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinbut_delay));

			/* shift size */
			prm->shift_size = (int)gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinbut_shiftsize));

			/* colors - fg and bg */
			gtk_color_button_get_color(GTK_COLOR_BUTTON(fg_color_but), &prm->fg_color);
			prm->fg_color_alpha = gtk_color_button_get_alpha(GTK_COLOR_BUTTON(fg_color_but));
			gtk_color_button_get_color(GTK_COLOR_BUTTON(bg_color_but), &prm->bg_color);
			prm->bg_color_alpha = gtk_color_button_get_alpha(GTK_COLOR_BUTTON(bg_color_but));

			/* font */
			str_n_cpy(prm->font_n_s, (char *)gtk_font_button_get_font_name(
						GTK_FONT_BUTTON(font_but)), 68);

			/* window x, y, w, h */
			prm->win_x = (gint)gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinbut_winx));
			prm->win_y = (gint)gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinbut_winy));
			prm->win_w = (gint)gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinbut_winw));

			/* if win_h is != 0, it will override requested font size with computed one */
			prm->win_h = (gint)gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinbut_winh));
			if (prm->win_h > 0 && prm->win_h < DRWA_HEIGHT_MIN)
				prm->win_h = DRWA_HEIGHT_MIN;

			/* window decoration */
			if ((gboolean)gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbut_windec)))
				prm->windec = 'y';
			else
				prm->windec = 'n';

			/* always-on-top */
			if ((gboolean)gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbut_alwaysontop)))
				prm->always_on_top = 'y';
			else
				prm->always_on_top = 'n';

			/* window transparency */
			prm->win_transparency = gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinbut_wintransparency)) / 10;

			/* shadow */
			if ((gboolean)gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbut_shadow)))
				prm->shadow = 'y';
			else
				prm->shadow = 'n';
			prm->shadow_offset_x = (int)gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinbut_shadowoffsetx));
			prm->shadow_offset_y = (int)gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinbut_shadowoffsety));
			prm->shadow_fx = (int)gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinbut_shadowfx));

			/* line and cut line delimiters */
			str_n_cpy(prm->line_delimiter,
				(char *)gtk_entry_get_text(GTK_ENTRY(entry_linedel)), DELIMITER_MAXLEN);
			str_n_cpy(prm->cutline_delimiter,
				(char *)gtk_entry_get_text(GTK_ENTRY(entry_cutlinedel)), DELIMITER_MAXLEN);

			/* rss refresh */
			prm->rss_refresh = (int)gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinbut_rssrefresh));

			/* rss title */
			if ((gboolean)gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbut_rsstitle)))
				prm->rss_title = 'y';
			else
				prm->rss_title = 'n';
			str_n_cpy(prm->rss_title_delimiter,
				(char *)gtk_entry_get_text(GTK_ENTRY(entry_rsstitledel)),DELIMITER_MAXLEN);

			/* rss decription */
			if ((gboolean)gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbut_rssdes)))
				prm->rss_description = 'y';
			else
				prm->rss_description = 'n';
			str_n_cpy(prm->rss_description_delimiter,
				(char *)gtk_entry_get_text(GTK_ENTRY(entry_rssdesdel)), DELIMITER_MAXLEN);

			/* strip html tags */
			if ((gboolean)gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbut_striptags)))
				prm->strip_html_tags = 'y';
			else
				prm->strip_html_tags = 'n';

			/* homefeed */
			str_n_cpy(prm->homefeed,
				(char *)gtk_entry_get_text(GTK_ENTRY(entry_homefeed)), FILENAME_MAXLEN);

			/* open link cmd */
			str_n_cpy(prm->open_link_cmd,
				(char *)gtk_entry_get_text(GTK_ENTRY(entry_openlinkcmd)), FILENAME_MAXLEN);

			/* open link cmd */
			str_n_cpy(prm->open_link_args,
				(char *)gtk_entry_get_text(GTK_ENTRY(entry_openlinkargs)), FILENAME_MAXLEN);

			/* clock */
			if ((gboolean)gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radio_but1)))
				prm->clock = 'l';
			else if ((gboolean)gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radio_but2)))
				prm->clock = 'r';
			else
				prm->clock = 'n';

			str_n_cpy(prm->clock_font_n_s, (char *)gtk_font_button_get_font_name(
					GTK_FONT_BUTTON(clock_font_but)), FONT_MAXLEN);

			gtk_color_button_get_color(GTK_COLOR_BUTTON(clock_fg_color_but),
								&prm->clock_fg_color);
			prm->clock_fg_color_alpha = gtk_color_button_get_alpha(
						GTK_COLOR_BUTTON(clock_fg_color_but));
			gtk_color_button_get_color(GTK_COLOR_BUTTON(clock_bg_color_but),
								&prm->clock_bg_color);
			prm->clock_bg_color_alpha = gtk_color_button_get_alpha(
						GTK_COLOR_BUTTON(clock_bg_color_but));

			/* special chars */
			if ((gboolean)gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbut_spchars)))
				prm->special_chars = 'y';
			else
				prm->special_chars = 'n';
			prm->new_page_char = gtk_entry_get_text(GTK_ENTRY(entry_newpagech))[0];
			prm->tab_char = gtk_entry_get_text(GTK_ENTRY(entry_tabch))[0];

			/* we want a new pixmap so that changes will be effective... now! */
			if (response == GTK_RESPONSE_APPLY) {
				if (memcmp((const void *)prm, (const void *)prm_bak, \
						sizeof(Params)) != 0)
					env->reload_rq = TRUE;	/* compute_rq would skip to next pixmap if any */
				env->suspend_rq = FALSE;
			} else if (response == GTK_RESPONSE_YES) {	/* = save */
				env->reload_rq = TRUE;
				env->suspend_rq = FALSE;
				save_to_config_file(prm);
				break;
			} else if (response == GTK_RESPONSE_OK) {
				if (memcmp((const void *)prm, (const void *)prm_bak, \
						sizeof(Params)) != 0)
					env->reload_rq = TRUE;
				env->suspend_rq = FALSE;
				break;
			}
		}
	}
	free2(prm_bak);
	gtk_widget_destroy(dialog);
	check_main_win_always_on_top();
	return exit_status;
}
#endif
